@@ -26,7 +26,7 @@ using namespace cpp11::literals;
 class type_error : public std::exception {
  public:
   type_error(int expected, int actual) : expected_(expected), actual_(actual) {}
-  virtual const char* what() const noexcept {
+  virtual const char* what() const noexcept override {
     snprintf(str_, 64, "Invalid input type, expected '%s' actual '%s'",
              Rf_type2char(expected_), Rf_type2char(actual_));
     return str_;
@@ -56,7 +56,7 @@ class r_vector {
   typedef T* pointer;
   typedef T& reference;
 
-  r_vector() = default;
+  r_vector() noexcept = default;
 
   r_vector(SEXP data);
 
@@ -103,36 +103,43 @@ class r_vector {
   };
 
   r_vector(const writable::r_vector<T>& rhs) : r_vector(static_cast<SEXP>(rhs)) {}
+  r_vector(named_arg) = delete;
 
   bool is_altrep() const;
 
+  bool named() const;
+
   R_xlen_t size() const;
 
   operator SEXP() const;
 
   operator sexp() const;
 
+  bool empty() const;
+
   /// Provide access to the underlying data, mainly for interface
   /// compatibility with std::vector
   SEXP data() const;
 
-  sexp attr(const char* name) const {
+  const sexp attr(const char* name) const {
     return SEXP(attribute_proxy<r_vector<T>>(*this, name));
   }
 
-  sexp attr(const std::string& name) const {
+  const sexp attr(const std::string& name) const {
     return SEXP(attribute_proxy<r_vector<T>>(*this, name.c_str()));
   }
 
-  sexp attr(SEXP name) const { return SEXP(attribute_proxy<r_vector<T>>(*this, name)); }
+  const sexp attr(SEXP name) const {
+    return SEXP(attribute_proxy<r_vector<T>>(*this, name));
+  }
 
   r_vector<r_string> names() const {
-    SEXP nms = SEXP(attribute_proxy<r_vector<T>>(*this, R_NamesSymbol));
+    SEXP nms = SEXP(Rf_getAttrib(data_, R_NamesSymbol));
     if (nms == R_NilValue) {
       return r_vector<r_string>();
     }
 
-    return nms;
+    return r_vector<r_string>(nms);
   }
 
   class const_iterator {
@@ -145,7 +152,7 @@ class r_vector {
 
     const_iterator(const r_vector* data, R_xlen_t pos);
 
-    inline const_iterator& operator+(R_xlen_t pos);
+    inline const_iterator operator+(R_xlen_t pos);
     inline ptrdiff_t operator-(const const_iterator& other) const;
 
     inline const_iterator& operator++();
@@ -271,18 +278,17 @@ class r_vector : public cpp11::r_vector<T> {
 
     using cpp11::r_vector<T>::const_iterator::operator!=;
 
-    inline iterator& operator+(R_xlen_t rhs);
+    inline iterator& operator+=(R_xlen_t rhs);
+    inline iterator operator+(R_xlen_t rhs);
   };
 
-  r_vector() = default;
+  r_vector() noexcept = default;
   r_vector(const SEXP& data);
   r_vector(SEXP&& data);
   r_vector(const SEXP& data, bool is_altrep);
   r_vector(SEXP&& data, bool is_altrep);
   r_vector(std::initializer_list<T> il);
   r_vector(std::initializer_list<named_arg> il);
-  r_vector(std::initializer_list<const char*> il);
-  r_vector(std::initializer_list<std::string> il);
 
   template <typename Iter>
   r_vector(Iter first, Iter last);
@@ -290,7 +296,7 @@ class r_vector : public cpp11::r_vector<T> {
   template <typename V, typename W = has_begin_fun<V>>
   r_vector(const V& obj);
 
-  r_vector(const R_xlen_t size);
+  explicit r_vector(const R_xlen_t size);
 
   ~r_vector();
 
@@ -379,6 +385,11 @@ inline bool r_vector<T>::is_altrep() const {
   return is_altrep_;
 }
 
+template <typename T>
+inline bool r_vector<T>::named() const {
+  return Rf_getAttrib(data_, R_NamesSymbol) != R_NilValue;
+}
+
 template <typename T>
 inline R_xlen_t r_vector<T>::size() const {
   return length_;
@@ -389,6 +400,11 @@ inline r_vector<T>::operator SEXP() const {
   return data_;
 }
 
+template <typename T>
+inline bool r_vector<T>::empty() const {
+  return (!(this->size() > 0));
+}
+
 template <typename T>
 inline r_vector<T>::operator sexp() const {
   return data_;
@@ -486,13 +502,11 @@ inline ptrdiff_t r_vector<T>::const_iterator::operator-(
 }
 
 template <typename T>
-inline typename r_vector<T>::const_iterator& r_vector<T>::const_iterator::operator+(
+inline typename r_vector<T>::const_iterator r_vector<T>::const_iterator::operator+(
     R_xlen_t rhs) {
-  pos_ += rhs;
-  if (data_->is_altrep() && pos_ >= block_start_ + length_) {
-    fill_buf(pos_);
-  }
-  return *this;
+  auto it = *this;
+  it += rhs;
+  return it;
 }
 
 template <typename T>
@@ -611,7 +625,7 @@ inline typename r_vector<T>::iterator& r_vector<T>::iterator::operator++() {
 }
 
 template <typename T>
-inline typename r_vector<T>::iterator& r_vector<T>::iterator::operator+(R_xlen_t rhs) {
+inline typename r_vector<T>::iterator& r_vector<T>::iterator::operator+=(R_xlen_t rhs) {
   pos_ += rhs;
   if (data_.is_altrep() && pos_ >= block_start_ + length_) {
     fill_buf(pos_);
@@ -619,6 +633,13 @@ inline typename r_vector<T>::iterator& r_vector<T>::iterator::operator+(R_xlen_t
   return *this;
 }
 
+template <typename T>
+inline typename r_vector<T>::iterator r_vector<T>::iterator::operator+(R_xlen_t rhs) {
+  auto it = *this;
+  it += rhs;
+  return it;
+}
+
 template <typename T>
 inline typename r_vector<T>::iterator r_vector<T>::begin() const {
   return iterator(*this, 0);
@@ -674,7 +695,7 @@ inline r_vector<T>::r_vector(const V& obj) : r_vector() {
 }
 
 template <typename T>
-inline r_vector<T>::r_vector(R_xlen_t size) : r_vector() {
+inline r_vector<T>::r_vector(const R_xlen_t size) : r_vector() {
   resize(size);
 }
 
@@ -768,9 +789,7 @@ inline r_vector<T>::r_vector(const r_vector<T>& rhs)
 
 template <typename T>
 inline r_vector<T>::r_vector(r_vector<T>&& rhs)
-    : cpp11::r_vector<T>(rhs),
-      protect_(preserved.insert(data_)),
-      capacity_(rhs.capacity_) {
+    : cpp11::r_vector<T>(rhs), protect_(rhs.protect_), capacity_(rhs.capacity_) {
   rhs.data_ = R_NilValue;
   rhs.protect_ = R_NilValue;
 }
@@ -868,17 +887,32 @@ inline void r_vector<T>::clear() {
   length_ = 0;
 }
 
+inline SEXP truncate(SEXP x, R_xlen_t length, R_xlen_t capacity) {
+#if R_VERSION >= R_Version(3, 4, 0)
+  SETLENGTH(x, length);
+  SET_TRUELENGTH(x, capacity);
+  SET_GROWABLE_BIT(x);
+#else
+  x = safe[Rf_lengthgets](x, length);
+#endif
+  return x;
+}
+
 template <typename T>
 inline r_vector<T>::operator SEXP() const {
+  auto* p = const_cast<r_vector<T>*>(this);
+  if (data_ == R_NilValue) {
+    p->resize(0);
+    return data_;
+  }
   if (length_ < capacity_) {
-#if R_VERSION >= R_Version(3, 4, 0)
-    SETLENGTH(data_, length_);
-    SET_TRUELENGTH(data_, capacity_);
-    SET_GROWABLE_BIT(data_);
-#else
-    auto* p = const_cast<r_vector<T>*>(this);
-    p->data_ = safe[Rf_lengthgets](data_, length_);
-#endif
+    p->data_ = truncate(p->data_, length_, capacity_);
+    SEXP nms = names();
+    auto nms_size = Rf_xlength(nms);
+    if ((nms_size > 0) && (length_ < nms_size)) {
+      nms = truncate(nms, length_, capacity_);
+      names() = nms;
+    }
   }
   return data_;
 }
